/*
 * Copyright (C) 2011-2016 Gary Kramlich <grim@reaperworld.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <https://www.gnu.org/licenses/>.
 */

#include <glib.h>

#include <gplugin.h>
#include <gplugin-native.h>

/******************************************************************************
 * Tests
 *****************************************************************************/
static void
test_unresolved_symbol(void)
{
	GPluginLoader *loader = NULL;
	GPluginManager *manager = NULL;

	manager = g_object_new(GPLUGIN_TYPE_MANAGER, NULL);

	loader = gplugin_native_loader_new();
	gplugin_manager_register_loader(manager, loader, NULL);
	g_clear_object(&loader);

	g_test_expect_message(
		G_LOG_DOMAIN,
		G_LOG_LEVEL_WARNING,
		"*some_unresolved_symbol*");

	gplugin_manager_remove_paths(manager);
	gplugin_manager_append_path(manager, TEST_UNRESOLVED_SYMBOL_DIR);
	gplugin_manager_refresh(manager);

	g_test_assert_expected_messages();

	g_assert_finalize_object(manager);
}

/******************************************************************************
 * Main
 *****************************************************************************/
int
main(int argc, char **argv)
{
	g_test_init(&argc, &argv, NULL);

	/* test the load on query flag */
	g_test_add_func(
		"/loaders/native/unresolved-symbol",
		test_unresolved_symbol);

	return g_test_run();
}
